"""
GXN: Generalizable Gene Self-Expressive Network

This package allows to infer Generalizable Gene Self-Expressive Network through
OMP and ElasticNet.
"""
__author__ = "Sergio Peignier"
__copyright__ = "Copyright 2019, The GXN Project"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "Sergio Peignier"
__email__ = "sergio.peignier@insa-lyon.fr"
__status__ = "pre-alpha"
